/* This file is part of the Zenipex Library.
* Copyleft (C) 2011 Mitchell Keith Bloch a.k.a. bazald
*
* This source file is simply under the public domain.
*/

#include <zenilib.h>

#include "Tutorial_State.h"

using namespace std;
using namespace Zeni;

Tutorial_State::Tutorial_State()
: m_time_passed(0.0f),
m_max_time_step(1.0f / 20.0f), // make the largest physics step 1/20 of a second
m_max_time_steps(10.0f) // allow no more than 10 physics steps per frame
{
  // If our game has no real time component in real life, allow the user to pause the game.
  // This would be a BAD idea in a networked multiplayer mode for a game.
  // (Or at the very least, this would be the wrong way to do it.)
  set_pausable(true);
}

void Tutorial_State::on_push() {
  get_Window().set_mouse_state(Window::MOUSE_HIDDEN);
}

void Tutorial_State::on_pop() {
  get_Controllers().reset_vibration_all();
}

void Tutorial_State::on_cover() {
  get_Controllers().reset_vibration_all();
}

void Tutorial_State::on_controller_axis(const SDL_ControllerAxisEvent &event) {
	switch(event.axis) {
		case SDL_CONTROLLER_AXIS_LEFTX:
			m_controls.left = event.value < -16000;
			m_controls.right = event.value > 16000;
			break;

		case SDL_CONTROLLER_AXIS_LEFTY:
			m_controls.up = event.value < -16000;
			m_controls.down = event.value > 16000;
			break;

    default:
      break;
	}
}

void Tutorial_State::on_controller_button(const SDL_ControllerButtonEvent &event) {
  if(event.button == SDL_CONTROLLER_BUTTON_BACK && event.state == SDL_PRESSED)
    get_Game().push_Popup_Menu_State();
}

void Tutorial_State::on_key(const SDL_KeyboardEvent &event) {
	switch(event.keysym.sym) {
		case SDLK_LEFT:
			m_controls.left = event.type == SDL_KEYDOWN;
			break;

		case SDLK_RIGHT:
			m_controls.right = event.type == SDL_KEYDOWN;
			break;

		case SDLK_UP:
			m_controls.up = event.type == SDL_KEYDOWN;
			break;

		case SDLK_DOWN:
			m_controls.down = event.type == SDL_KEYDOWN;
			break;

		default:
			Gamestate_Base::on_key(event);
	}
}

void Tutorial_State::perform_logic() {
  {// Update time_passed
    const Time temp_time = get_Timer().get_time();
    m_time_passed += temp_time.get_seconds_since(m_current_time);
    m_current_time = temp_time;
  }

  /* Shrink time passed to an upper bound
   *
   * If your program is ever paused by your computer for 10 
   * minutes and then allowed to continue running, you don't want 
   * it to pause at the physics loop for another 10 minutes.
   */
  if(m_time_passed / m_max_time_step > m_max_time_steps)
    m_time_passed = m_max_time_steps * m_max_time_step;

  /* Constant time-step physics update
   *
   * This type of update increases the stability of your program 
   * in scenarios where the time step could allow things to move 
   * further than you expect, potentially jumping though walls or 
   * falling through floors... that sort of thing.  Keep the 
   * max_time_step small and you are safe.
   */
  while(m_time_passed > m_max_time_step) {
    step(m_max_time_step);
    m_time_passed -= m_max_time_step;
  }

  /* Simple physics update
   *
   * This type of update could be the only update you do if you 
   * don't have concerns with respect to jumping though walls, 
   * etc...
   *
   * Otherwise, if you have the other update, this serves to 
   * improve the smoothness of your animations.  With the other 
   * version alone, things can jerk a bit from frame to frame.
   *
   * To really see the improvement, turn up multisampling and try 
   * it with this section (A) commented out, and then (B) not
   * commented out.
   */
  {
    step(m_time_passed);
    m_time_passed = 0.0f;
  }
}

void Tutorial_State::step(const float &time_step) {
  m_rect.position.x += m_rect.max_velocity.i * time_step *
                       (m_controls.right - m_controls.left);
  m_rect.position.y += m_rect.max_velocity.j * time_step *
                       (m_controls.down - m_controls.up);
}

void Tutorial_State::render() {
  Video &vr = get_Video();

  vr.set_2d();

  Vertex2f_Texture p0(m_rect.position,                                                               Point2f(0.0f, 0.0f));
  Vertex2f_Texture p1(Point2f(m_rect.position.x,                 m_rect.position.y + m_rect.size.j), Point2f(0.0f, 1.0f));
  Vertex2f_Texture p2(Point2f(m_rect.position.x + m_rect.size.i, m_rect.position.y + m_rect.size.j), Point2f(1.0f, 1.0f)); 
  Vertex2f_Texture p3(Point2f(m_rect.position.x + m_rect.size.i, m_rect.position.y),                 Point2f(1.0f, 0.0f));
  Material material("logo");

	Quadrilateral<Vertex2f_Texture> quad(p0, p1, p2, p3);
  quad.fax_Material(&material);

	vr.render(quad);
}
